library upi_payment;

import 'dart:async';
import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:http/http.dart' as http;
import 'package:flutter_easyloading/flutter_easyloading.dart';

///Single case
abstract class PaymentModuleObserver {
  paymentResponse(PaymentModule observable,String? notifyName ,String? planId, Map map);
}

class PaymentModule {
  factory PaymentModule() {
    return _getInstance();
  }

  static PaymentModule _instance = PaymentModule._();

  static PaymentModule get instance => _instance;

  static PaymentModule _getInstance() {
    return _instance;
  }

  late List<PaymentModuleObserver> observers;

  PaymentModule._() {
    observers = [];
  }

  int length() {
    return observers.length;
  }

  ///Add observer
  addPaymentModuleObserver(PaymentModuleObserver observer) {
    if (!_checkContains(observer)) {
      observers.add(observer);
    }
  }

  ///重复添加
  ///add Repeatedly
  addRepeatedlyPaymentModuleObserver(PaymentModuleObserver observer) {
    observers.add(observer);
  }

  ///Remove viewers when the page is not in use
  removePaymentModuleObserver(PaymentModuleObserver observer) {
    observers.remove(observer);
  }

  ///Used in the page that sent the notification
  /// stateNames state name to update
  /// Map notification data, can be null
  notifyPaymentModuleObservers(List<String> stateNames, {String? notifyName, String? planId,required Map map}) {
    if (stateNames.isEmpty) {
      for (int i = observers.length - 1; i >= 0; i--) {
        observers[i].paymentResponse(this, notifyName, planId,map);
      }
    } else {
      stateNames.forEach((stateName) {
        for (int i = observers.length - 1; i >= 0; i--) {
          if (stateName == observers[i].runtimeType.toString()) {
            observers[i].paymentResponse(this,notifyName, planId, map);
          }
        }
      });
    }
  }

  ///Check if it has been added to prevent the removal of remove PaymentModuleObserver in the dispose method, an error occurs
  bool _checkContains(PaymentModuleObserver observer) {
    for (int i = 0; i < observers.length; i++) {
      if (observers[i].runtimeType.toString() == (observer.runtimeType.toString())) {
        observers.removeAt(i);
        observers.insert(i, observer);
        return true;
      }
    }
    return false;
  }
}


class PaymentApi {
  static const String paymentBaseUrl = "https://www.bizzbrains.com/bizzbrain-payment/api/";

  static const String collectPay = paymentBaseUrl + "collectpay";
  static const String qrCode = paymentBaseUrl + "qrcode";
  static const String getTransactionDetail = paymentBaseUrl + "getTransaction";
  static const String cancelTransaction = paymentBaseUrl + "cancelTransaction";
}

class PaymentView extends StatefulWidget {
  final String price;
  final String orderId;

  const PaymentView(this.price, this.orderId, {Key? key}) : super(key: key);

  @override
  State<PaymentView> createState() => _PaymentViewState();
}

class _PaymentViewState extends State<PaymentView> {
  String qrLink = "";

  final qrdataFeed = TextEditingController();
  Timer? timer;
  final upiTextController = TextEditingController();
  bool _validate = false;
  String merchantTranId = "";
  String transactionStatus = "";

  String apiKey = "bizz_live_BFOfjeKGinPPwd";
  String apiSecret = "f8561ab2-e7d1-4770-bbc6-6663850cd0cc";

  late BuildContext mainContext;

  bool isLoading = false;

  Map<String, dynamic> paymentResponse = {};

  static const countdownDuration = Duration(minutes: 10);
  Duration duration = Duration();
  Timer? countDownTimer;

  String countDownTime = "";
  bool countDown = true;
  bool isTimeout = false;

  @override
  void initState() {
    // TODO: implement initState
    super.initState();
    reset();
  }

  @override
  void dispose() {
    countDown = false;
    cancleTimer();
    countDownTimer?.cancel();
    super.dispose();
  }

  void reset() {
    if (countDown) {
      setState(() => duration = countdownDuration);
    } else {
      setState(() => duration = Duration());
    }
  }

  void startTimer() {
    countDownTimer = Timer.periodic(Duration(seconds: 1), (Timer t) => addTime());
  }

  void addTime() {
    final addSeconds = countDown ? -1 : 1;
    setState(() {
      final seconds = duration.inSeconds + addSeconds;
      if (seconds < 0) {
        isTimeout = true;
        stopTimer();
        cancleTimer();
        cancelTransaction(context);
      } else {
        duration = Duration(seconds: seconds);
        String twoDigits(int n) => n.toString().padLeft(2, '0');
        setState1(() {
          countDownTime = "${twoDigits(duration.inMinutes.remainder(60))}:${twoDigits(duration.inSeconds.remainder(60))}";
        });
      }
    });
  }

  void stopTimer({bool resets = true}) {
    if (resets) {
      reset();
    }
    setState(() => countDownTimer?.cancel());
  }

  @override
  Widget build(BuildContext context) {
    Size size = MediaQuery.of(context).size;
    mainContext = context;
    return Scaffold(
      resizeToAvoidBottomInset: false,
      body: Stack(
        children: [
          Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              Container(
                width: MediaQuery.of(context).size.width,
                color: Color(0xffFEF5E6),
                child: Column(
                  mainAxisAlignment: MainAxisAlignment.center,
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: <Widget>[
                    Row(
                      children: [
                        Expanded(
                          child: Container(
                            margin: const EdgeInsets.only(left: 20, top: 45),
                            child: const Text("Bizzbrain",
                                style: TextStyle(
                                    fontSize: 20,
                                    fontWeight: FontWeight.w700,
                                    fontFamily: "Relevant",
                                    color: Color(0xffC24E02))),
                          ),
                        ),
                        GestureDetector(
                          onTap: () async {
                            if (transactionStatus == "SUCCESS") {
                              PaymentModule.instance.observers.forEach((element) {
                                element.paymentResponse(PaymentModule(), transactionStatus, widget.orderId, paymentResponse);
                              });
                              Navigator.pop(context, paymentResponse);
                            } else {
                              Navigator.pop(context);
                            }
                          },
                          child: const Padding(
                            padding: EdgeInsets.only(left: 20, top: 45, right: 20),
                            child: Icon(
                              Icons.close,
                              size: 20,
                              color: Color(0xffC24E02),
                            ),
                          ),
                        ),
                      ],
                    ),
                    Container(
                      margin: const EdgeInsets.only(left: 20),
                      child: Text(
                        "Order ${widget.orderId}",
                        style: const TextStyle(
                            fontSize: 14, fontWeight: FontWeight.w700, fontFamily: "Relevant", color: Color(0xffD4824D)),
                      ),
                    ),
                    Container(
                      margin: const EdgeInsets.symmetric(horizontal: 20, vertical: 5),
                      child: Text(
                        "₹ ${widget.price}",
                        style: const TextStyle(
                            fontSize: 22, fontWeight: FontWeight.w700, fontFamily: "Relevant", color: Color(0xffC24E02)),
                      ),
                    ),
                  ],
                ),
              ),
              Container(
                height: MediaQuery.of(context).size.height * 0.05,
                width: MediaQuery.of(context).size.width,
                color: Color(0xffF87D42),
              ),
              Container(
                margin: const EdgeInsets.only(left: 20, top: 10),
                child: const Text(
                  "PAY USING UPI ID",
                  style: TextStyle(fontSize: 14, fontWeight: FontWeight.w400, fontFamily: "Relevant", color: Color(0xff656565)),
                ),
              ),
              upiView(context),
              Container(
                margin: const EdgeInsets.only(left: 20, top: 10),
                child: const Text(
                  "PAY USING QR CODE",
                  style: TextStyle(fontSize: 14, fontWeight: FontWeight.w400, fontFamily: "Relevant", color: Color(0xff656565)),
                ),
              ),
              qrCode(context),
            ],
          ),
          Visibility(
            visible: isLoading,
            child: const Center(
              child: CircularProgressIndicator(color: Color(0xffFA9905)),
            ),
          )
        ],
      ),
      bottomNavigationBar: InkWell(
        onTap: () async {

          if (upiTextController.text.trim() == "") {
          } else {

            FocusManager.instance.primaryFocus?.unfocus();
            bool isSuccess = await collectPay();
            setState(() {
              if (isSuccess) {
                startTimerggetTranction(false);
                startTimer();
                paymentProgress(context);
              }
            });
          }
        },
        child: Container(
          alignment: Alignment.center,
          decoration: const BoxDecoration(
            gradient: LinearGradient(
              colors: [Color(0xffFF5200), Color(0xffC24E02)],
              begin: FractionalOffset.topCenter,
              end: FractionalOffset.bottomCenter,
            ),
          ),
          width: MediaQuery.of(context).size.width,
          height: MediaQuery.of(context).size.height * 0.085,
          child: Text(
            "PAY ₹ ${widget.price}",
            style: TextStyle(fontSize: 20, fontWeight: FontWeight.w600, fontFamily: "Relevant", color: const Color(0xffFFFFFF)),
          ),
        ),
      ),
    );
  }

  Widget qrCode(BuildContext context) {
    return Container(
      decoration: BoxDecoration(
        border: Border.all(color: const Color(0xffFDEACC), width: 1),
        color: const Color(0xffFFFCF9),
        borderRadius: const BorderRadius.all(Radius.circular(8)),
      ),
      // decoration: BoxDecoration(border: Border.all(color: Colors.grey, width: 1)),
      margin: const EdgeInsets.fromLTRB(20, 5, 20, 5),
      child: GestureDetector(
        onTap: () {
          genrateQRcode(context);
        },
        child: Container(
          width: MediaQuery.of(context).size.width,
          margin: const EdgeInsets.symmetric(vertical: 10),
          child: Padding(
            padding: const EdgeInsets.symmetric(horizontal: 20, vertical: 10),
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: const [
                Text(
                  "Show QR Code",
                  style:
                  TextStyle(fontSize: 16, fontWeight: FontWeight.w700, fontFamily: "Relevant", color: Color(0xff727272)),
                ),
                Text(
                  "Scan the QR code using your UPI app",
                  style:
                  TextStyle(fontSize: 14, fontWeight: FontWeight.w400, fontFamily: "Relevant", color: Color(0xff727272)),
                ),
              ],
            ),
          ),
        ),
      ),
    );
  }

  String? get _errorText {
    // at any time, we can get the text from _controller.value.text
    final text = upiTextController.value.text;
    // Note: you can do your own custom validation here
    // Move this logic this outside the widget for more testable code
    if (text.isEmpty) {
      return 'Can\'t be empty';
    }
    // return null if the text is valid
    return null;
  }

  Widget upiView(BuildContext context) {
    return Container(
      decoration: BoxDecoration(
        border: Border.all(color: const Color(0xffFA9905), width: 1),
        color: const Color(0xffFEF5E6),
        borderRadius: const BorderRadius.all(Radius.circular(8)),
      ),
      margin: const EdgeInsets.fromLTRB(20, 5, 20, 5),
      // width: MediaQuery.of(context).size.width,
      child: Row(
        crossAxisAlignment: CrossAxisAlignment.end,
        children: [
          Expanded(
            child: Container(
              margin: const EdgeInsets.all(10),
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  const Padding(
                    padding: EdgeInsets.symmetric(horizontal: 10.0),
                    child: Text(
                      "UPI ID",
                      style: TextStyle(
                          fontSize: 16, fontWeight: FontWeight.w700, fontFamily: "Relevant", color: Color(0xff727272)),
                    ),
                  ),
                  const Padding(
                    padding: const EdgeInsets.symmetric(horizontal: 10.0),
                    child: Text(
                      "Google Pay,BHIM, PhonePe & More",
                      style: TextStyle(
                          fontSize: 16, fontWeight: FontWeight.w400, fontFamily: "Relevant", color: Color(0xff727272)),
                    ),
                  ),
                  Padding(
                    padding: const EdgeInsets.symmetric(horizontal: 10.0),
                    child: Row(
                      children: [
                        Expanded(
                          child: TextField(
                            controller: upiTextController,

                            cursorColor: const Color(0xffFA9905),
                            decoration: InputDecoration(
                              enabledBorder: const UnderlineInputBorder(
                                  borderSide: BorderSide(color: Color(0xffFA9905))
                              ),
                              focusedBorder: const UnderlineInputBorder(
                                  borderSide: BorderSide(color: Color(0xffFA9905))
                              ),
                              errorText: _errorText,
                              border: const UnderlineInputBorder(
                                borderSide: BorderSide(color: Color(0xffFA9905), width: 1.0),
                              ),
                              labelText: 'Enter your UPI ID',
                              labelStyle: const TextStyle(
                                  fontSize: 14,
                                  fontWeight: FontWeight.w400,
                                  fontFamily: "Relevant",
                                  color: Color(0xff727272)),
                            ),
                          ),
                        ),
                      ],
                    ),
                  ),
                ],
              ),
            ),
          ),
          // Padding(
          //   padding: const EdgeInsets.only(bottom: 16.0, right: 10),
          //   child: GestureDetector(
          //       onTap: () {
          //         print(upiTextController.value.text);
          //       },
          //       child: Container(
          //         padding: EdgeInsets.all(5),
          //         decoration: BoxDecoration(
          //             borderRadius: BorderRadius.all(Radius.circular(6)),
          //             color: Color("#FA9905")
          //         ),
          //         child: Text("Verify", style: TextStyle(
          //           fontSize: 14, fontWeight: FontWeight.w700, fontFamily: "Relevant", color: Color("#FFFFFF"),),
          //         ),
          //       )
          //   ),
          // ),
        ],
      ),
    );
  }

  void genrateQRcode(BuildContext context) {
    showModalBottomSheet(
        isDismissible: false,
        shape: const RoundedRectangleBorder(
            borderRadius: BorderRadius.vertical(
                top: Radius.circular(15)
            )
        ),
        context: context,
        builder: (BuildContext bc) {
          return WillPopScope(
            onWillPop: () async => false,
            child: Column(
              children: <Widget>[
                InkWell(
                  onTap: () {
                    countDownTimer?.cancel;
                    cancleTimer();
                    stopTimer();
                    cancelTransaction(context);
                    Navigator.pop(context);
                  },
                  child: const Align(
                    alignment: Alignment.centerRight,
                    child: Padding(
                      padding: const EdgeInsets.only(top: 15, right: 15),
                      child: Icon(
                        Icons.close,
                        size: 20,
                        color: Color(0xff000000),
                      ),
                    ),
                  ),
                ),
                const Padding(
                  padding: const EdgeInsets.all(15),
                  child: Text(
                    "Scan the QR using any UPI app on your phone like BHIM,PhonePe,Google Pay etc.",
                    style: TextStyle(
                      fontSize: 16,
                      fontWeight: FontWeight.w700,
                      fontFamily: "Relevant",
                      color: Color(0xff000000),
                    ),
                    textAlign: TextAlign.center,
                  ),
                ),
                Align(
                  child: Image.asset(
                    'assets/images/payment_option_qr.png',
                    alignment: Alignment.center,
                    height: 50,
                    width: MediaQuery.of(context).size.width * 0.45,
                  ),
                ),
                StatefulBuilder(builder: (BuildContext context, StateSetter setState /*You can rename this!*/) {
                  setState1 = setState;
                  return Text(countDownTime);
                }),
                Expanded(
                  child: Container(
                    alignment: Alignment.center,
                    child: FutureBuilder<String>(
                      future: getQr(), // async work
                      builder: (BuildContext context, AsyncSnapshot<String> snapshot) {
                        switch (snapshot.connectionState) {
                          case ConnectionState.waiting:
                            return const CircularProgressIndicator(color: Color(0xffFA9905));
                          default:
                            if (snapshot.hasError) {
                              return Text('Error: ${snapshot.error}');
                            } else {
                              return Center(
                                child: Image.network(
                                  snapshot.data.toString(),
                                  loadingBuilder: (BuildContext context, Widget child, ImageChunkEvent? loadingProgress) {
                                    if (loadingProgress == null) return child;
                                    return const Center(
                                      child: CircularProgressIndicator(color: Color(0xffFA9905)),
                                    );
                                  },
                                  fit: BoxFit.cover,
                                ),
                              );
                            }
                        }
                      },
                    ),
                  ),
                )
              ],
            ),
          );
        });

  }

  late StateSetter setState1;

  void paymentProgress(BuildContext context) {
    showModalBottomSheet(
        isDismissible: false,
        context: context,
        builder: (BuildContext bc) {
          return WillPopScope(
            onWillPop: () async => false,
            child: StatefulBuilder(builder: (BuildContext context, StateSetter setState /*You can rename this!*/) {
              setState1 = setState;
              return Wrap(
                children: <Widget>[
                  Padding(
                    padding: const EdgeInsets.all(15),
                    child: transactionStatus != "INIT"
                        ? Column(
                      mainAxisAlignment: MainAxisAlignment.center,
                      children: [
                        Center(
                          child: Padding(
                            padding: const EdgeInsets.all(15),
                            child: Text(
                              transactionStatus == "SUCCESS"
                                  ? "Transaction Successful."
                                  : transactionStatus == "CANCEL"
                                  ? "You have cancelled the payment"
                                  : transactionStatus == "FAILURE"
                                  ? "Payment Declined"
                                  : isTimeout
                                  ? "You may have cancelled the payment or there was a delay in response from UPI app."
                                  : "Something wrong about UPI Id",
                              style: const TextStyle(
                                fontSize: 16,
                                fontWeight: FontWeight.w700,
                                fontFamily: "Relevant",
                                color: Color(0xff000000),
                              ),
                            ),
                          ),
                        ),
                        GestureDetector(
                          onTap: () async {
                            if (isTimeout) {
                              cancleTimer();
                              bool isSuccess = await collectPay();
                              setState1(() {
                                if (isSuccess) {
                                  startTimerggetTranction(false);
                                  startTimer();
                                } else {
                                  EasyLoading.showToast("Something went wrong!");
                                }
                              });
                            } else {
                              if (transactionStatus == "SUCCESS" || transactionStatus == "CANCEL") {
                                cancleTimer();
                                stopTimer();
                                Future.delayed(const Duration(seconds: 1), () {
                                  PaymentModule.instance.observers.forEach((element) {
                                    element.paymentResponse(PaymentModule(), transactionStatus, widget.orderId, paymentResponse);
                                  });
                                });
                                Navigator.pop(context);
                                Navigator.pop(mainContext);
                              } else {
                                cancleTimer();
                                stopTimer();
                                Navigator.pop(context);
                              }
                            }
                          },
                          child: Container(
                            decoration: const BoxDecoration(
                              color: Color(0xffFA9905),
                              borderRadius: BorderRadius.all(Radius.circular(5)),
                            ),
                            padding: const EdgeInsets.all(8),
                            child: Text(
                              isTimeout ? "Retry" : 'Done',
                              style: const TextStyle(
                                fontSize: 16,
                                fontWeight: FontWeight.w700,
                                fontFamily: "Relevant",
                                color: Color(0xffFFFFFF),
                              ),
                            ),
                          ),
                        ),
                      ],
                    )
                        : Column(
                      mainAxisAlignment: MainAxisAlignment.center,
                      children: [
                        const Center(
                          child: Padding(
                            padding: const EdgeInsets.all(15),
                            child: Text(
                              "Your payment is being processed",
                              style: TextStyle(
                                fontSize: 16,
                                fontWeight: FontWeight.w700,
                                fontFamily: "Relevant",
                                color: Color(0xff000000),
                              ),
                            ),
                          ),
                        ),
                        Text(countDownTime),
                        const Padding(
                          padding: EdgeInsets.all(15),
                          child: Center(
                            child: CircularProgressIndicator(color: Color(0xffFA9905)),
                          ),
                        ),
                        GestureDetector(
                          onTap: () async {
                            cancleTimer();
                            stopTimer();
                            await cancelTransaction(context);
                          },
                          child: const Text(
                            'Cancel Payment',
                            style: TextStyle(
                              fontSize: 14,
                              fontWeight: FontWeight.w400,
                              fontFamily: "Relevant",
                              color: Color(0xff000000),
                            ),
                          ),
                        ),
                        Container(
                          width: 120,
                          height: 3,
                          color: const Color(0xffFA9905),
                        )
                      ],
                    ),
                  ),
                ],
              );
            }),
          );
        });
  }

  Future<bool> collectPay() async {
    setState(() {
      isLoading = true;
    });

    var isSuccess = false;

    final headers = {'Content-Type': 'application/json'};
    Map<String, dynamic> body = {
      'api_key': apiKey,
      'api_secrets': apiSecret,
      "payerVa": upiTextController.value.text,
      "amount": widget.price,
      "note": "Remark",
      "order_id": widget.orderId
    };

    String jsonBody = json.encode(body);
    final encoding = Encoding.getByName('utf-8');

    http.Response response = await http.post(
      Uri.parse(PaymentApi.collectPay),
      headers: headers,
      body: jsonBody,
      encoding: encoding,
    );

    Map<String, dynamic> data = json.decode(response.body);

    print("data -> $data");

    if (response.statusCode == 200) {
      if (data['status'] == 200) {
        setState(() {
          merchantTranId = data['data']['merchantTranId'];
          transactionStatus = "INIT";
        });
        isSuccess = true;
      } else {
        EasyLoading.showToast(data['message'], toastPosition: EasyLoadingToastPosition.bottom);
      }
    } else {
      EasyLoading.showToast(data['message'] == "" ? "Something went wrong. Please try again." : data['message'], toastPosition: EasyLoadingToastPosition.bottom);
    }

    setState(() {
      isLoading = false;
    });

    return isSuccess;
  }

  void cancleTimer() {
    timer?.cancel();
  }

  void startTimerggetTranction(bool isQrOrNot) {
    timer = Timer.periodic(Duration(seconds: 5), (Timer t) => getTransactionDetail(context,isQrOrNot));
  }

  Future getTransactionDetail(BuildContext context,bool isQrOrNot) async {
    final headers = {'Content-Type': 'application/json'};
    Map<String, dynamic> body = {'api_key': apiKey, 'api_secrets': apiSecret, "txn_id": merchantTranId};

    String jsonBody = json.encode(body);
    final encoding = Encoding.getByName('utf-8');

    http.Response response = await http.post(
      Uri.parse(PaymentApi.getTransactionDetail),
      headers: headers,
      body: jsonBody,
      encoding: encoding,
    );

    Map<String, dynamic> data = json.decode(response.body);

    print("detail -> $data");

    if (response.statusCode == 200) {
      if (data['status'] == 200) {
        setState(() {
          paymentResponse = data;
          transactionStatus = data['data']['status'];

          if(isQrOrNot){
            if(transactionStatus == "SUCCESS" || transactionStatus == "CANCEL"|| transactionStatus == "FAILURE"){
              EasyLoading.showToast(transactionStatus == "SUCCESS" ? "Payment Successful" : data['message'], toastPosition: EasyLoadingToastPosition.top);
              cancleTimer();
              stopTimer();
              setState1(() {});
              if (transactionStatus == "SUCCESS" || transactionStatus == "CANCEL") {

                Future.delayed(const Duration(seconds: 1), () {
                  PaymentModule.instance.observers.forEach((element) {
                    element.paymentResponse(PaymentModule(), transactionStatus, widget.orderId, paymentResponse);
                  });
                });

                Navigator.pop(context);
                Navigator.pop(mainContext);
              } else {
                Navigator.pop(context);
              }
            }
          } else {
            if (transactionStatus == "SUCCESS" || transactionStatus == "CANCEL"|| transactionStatus == "FAILURE") {
              cancleTimer();
              stopTimer();
              setState1(() {});
            }
          }
        });
      } else {
        EasyLoading.showToast(data['message'], toastPosition: EasyLoadingToastPosition.top);
      }
    } else {
      EasyLoading.showToast("Something went wrong. Please try again.", toastPosition: EasyLoadingToastPosition.top);
      throw Exception('Something went wrong');
    }
  }

  Future cancelTransaction(BuildContext context) async {
    final headers = {'Content-Type': 'application/json'};
    Map<String, dynamic> body = {'api_key': apiKey, 'api_secrets': apiSecret, "txn_id": merchantTranId};

    String jsonBody = json.encode(body);
    final encoding = Encoding.getByName('utf-8');

    http.Response response = await http.post(
      Uri.parse(PaymentApi.cancelTransaction),
      headers: headers,
      body: jsonBody,
      encoding: encoding,
    );

    Map<String, dynamic> data = json.decode(response.body);

    if (response.statusCode == 200) {
      if (data['status'] == 200) {
        setState(() {
          if (this.mounted) {
            setState1(() {
              transactionStatus = "CANCEL";
            });
          }
        });
      } else {
        EasyLoading.showToast(data['message'], toastPosition: EasyLoadingToastPosition.top);
      }
    } else {
      EasyLoading.showToast("Something went wrong. Please try again.", toastPosition: EasyLoadingToastPosition.top);
      throw Exception('Something went wrong');
    }
  }

  Future<String> getQr() async {
    var qrUrl = "";

    final headers = {'Content-Type': 'application/json'};
    Map<String, dynamic> body = {'api_key': apiKey, 'api_secrets': apiSecret, "amount": widget.price, "order_id": 1};

    String jsonBody = json.encode(body);
    final encoding = Encoding.getByName('utf-8');

    http.Response response = await http.post(
      Uri.parse(PaymentApi.qrCode),
      headers: headers,
      body: jsonBody,
      encoding: encoding,
    );

    Map<String, dynamic> data = json.decode(response.body);

    if (response.statusCode == 200) {
      if (data['status'] == 200) {
        merchantTranId = data['data']['merchantTranId'];
        qrUrl = data['data']['qr_code'];
        setState(() {
          setState1(() {
            startTimerggetTranction(true);
            startTimer();
          });
        });

      } else {
        EasyLoading.showToast(data['message'], toastPosition: EasyLoadingToastPosition.top);
      }
    } else {
      EasyLoading.showToast(data['message'], toastPosition: EasyLoadingToastPosition.top);
    }

    return qrUrl;
  }
}